"""
 	 The MIT License (MIT)

 	 Copyright (c) 2014 GeekGao

 	 Permission is hereby granted, free of charge, to any person obtaining a copy
 	 of this software and associated documentation files (the "Software"), to deal
 	 in the Software without restriction, including without limitation the rights
 	 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 	 copies of the Software, and to permit persons to whom the Software is
 	 furnished to do so, subject to the following conditions:

 	 The above copyright notice and this permission notice shall be included in
 	 all copies or substantial portions of the Software.

 	 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 	 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 	 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 	 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 	 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 	 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 	 THE SOFTWARE.
 	 """

import random

ALL_ENG_NAMES = [
    "fAaliyah", "mAaron", "fAarushi", "fAbagail", "fAbbey", "fAbbi", "fAbbie", "fAbby", "mAbdul", "mAbdullah"
    , "mAbe", "mAbel", "fAbi", "fAbia", "fAbigail", "mAbraham", "mAbram", "fAbrianna", "mAbriel", "fAbrielle"
    , "fAby", "fAcacia", "mAce", "fAda", "fAdalia", "fAdalyn", "mAdam", "mAdan", "fAddie", "mAddison"
    , "fAddison", "mAde", "fAdelaide", "fAdele", "fAdelene", "fAdelia", "fAdelina", "fAdeline", "mAden", "mAdnan"
    , "mAdonis", "fAdreanna", "mAdrian", "fAdriana", "fAdrianna", "fAdrianne", "mAdriel", "fAdrienne", "fAerona",
    "fAgatha"
    , "fAggie", "fAgnes", "mAhmad", "mAhmed", "fAida", "mAidan", "mAiden", "fAileen", "fAilsa", "fAimee"
    , "fAine", "fAinsleigh", "fAinsley", "mAinsley", "fAisha", "fAisling", "fAislinn", "mAjay", "mAl", "mAlain"
    , "fAlaina", "mAlan", "fAlana", "fAlanis", "fAlanna", "fAlannah", "fAlaska", "mAlastair", "fAlayah", "fAlayna"
    , "fAlba", "mAlbert", "fAlberta", "mAlberto", "mAlbie", "mAlden", "mAldo", "fAleah", "mAlec", "fAlecia"
    , "fAleisha", "fAlejandra", "mAlejandro", "mAlen", "fAlena", "mAlesandro", "fAlessandra", "fAlessia", "mAlex",
    "fAlex"
    , "fAlexa", "mAlexander", "fAlexandra", "fAlexandria", "fAlexia", "fAlexis", "mAlexis", "fAlexus", "mAlfie",
    "mAlfonso"
    , "mAlfred", "mAlfredo", "mAli", "fAli", "fAlia", "fAlice", "fAlicia", "fAlina", "fAlisa", "fAlisha"
    , "fAlison", "fAlissa", "mAlistair", "fAlivia", "fAliyah", "fAliza", "fAlize", "fAlka", "mAllan", "mAllen"
    , "fAllie", "fAllison", "fAlly", "fAllyson", "fAlma", "fAlondra", "mAlonzo", "mAloysius", "mAlphonso", "mAlton"
    , "mAlvin", "fAlycia", "fAlyshialynn", "fAlyson", "fAlyssa", "fAlyssia", "fAmalia", "fAmanda", "fAmani", "fAmara"
    , "mAmari", "fAmari", "fAmaris", "fAmaya", "fAmber", "fAmberly", "fAmelia", "fAmelie", "fAmerica", "fAmethyst"
    , "fAmie", "fAmina", "mAmir", "fAmirah", "mAmit", "fAmity", "mAmos", "fAmy", "fAmya", "fAna"
    , "fAnabel", "fAnabelle", "fAnahi", "fAnais", "fAnamaria", "mAnand", "fAnanya", "fAnastasia", "mAnderson", "fAndie"
    , "mAndre", "fAndrea", "mAndreas", "mAndres", "mAndrew", "fAndromeda", "mAndy", "mAngel", "fAngel", "fAngela"
    , "fAngelia", "fAngelica", "fAngelina", "fAngeline", "fAngelique", "mAngelo", "fAngie", "mAngus", "fAnika", "fAnisa"
    , "fAnita", "fAniya", "fAniyah", "fAnjali", "fAnn", "fAnna", "fAnnabel", "fAnnabella", "fAnnabelle", "fAnnabeth"
    , "fAnnalisa", "fAnnalise", "fAnne", "fAnneke", "fAnnemarie", "fAnnette", "fAnnie", "fAnnika", "fAnnmarie", "mAnsel"
    , "mAnson", "fAnthea", "mAnthony", "fAntoinette", "mAnton", "fAntonia", "mAntonio", "mAntony", "fAnuja", "fAnusha"
    , "fAnushka", "fAnya", "fAoibhe", "fAoibheann", "fAoife", "fAphrodite", "mApollo", "fApple", "fApril", "fAqua"
    , "fArabella", "fArabelle", "mAran", "mArcher", "mArchie", "mAri", "fAria", "fAriadne", "fAriana", "fArianna"
    , "fArianne", "fAriel", "fAriella", "fArielle", "fArisha", "mArjun", "fArleen", "fArlene", "fArlette", "mArlo"
    , "mArman", "mArmando", "mArnold", "mAron", "mArran", "mArrie", "mArt", "fArtemis", "mArthur", "mArturo"
    , "mArun", "fArwen", "mArwin", "fArya", "mAsa", "mAsad", "mAsh", "fAsha", "fAshanti", "mAshby"
    , "mAsher", "fAshlee", "fAshleigh", "fAshley", "mAshley", "fAshlie", "fAshlyn", "fAshlynn", "mAshton", "fAshton"
    , "fAshvini", "fAsia", "fAsma", "mAspen", "fAspen", "mAston", "fAstrid", "mAthan", "fAthena", "fAthene"
    , "mAtticus", "fAubreanna", "fAubree", "fAubrey", "mAubrey", "fAudra", "fAudrey", "fAudrina", "mAudwin", "mAugust"
    , "fAugustina", "mAugustus", "fAurelia", "fAurora", "mAusten", "mAustin", "fAutumn", "fAva", "fAvalon", "mAvery"
    , "fAvery", "fAvril", "mAxel", "fAya", "mAyaan", "fAyana", "fAyanna", "mAyden", "fAyesha", "fAyisha"
    , "fAyla", "fAzalea", "fAzaria", "fAzariah", "fBailey", "mBailey", "mBarack", "fBarbara", "fBarbie", "mBarclay"
    , "mBarnaby", "mBarney", "mBarrett", "mBarron", "mBarry", "mBart", "mBartholomew", "mBasil", "mBastian", "mBaxter"
    , "mBay", "fBay", "fBaylee", "mBaylor", "fBea", "mBear", "fBeatrice", "fBeatrix", "mBeau", "fBecca"
    , "fBeccy", "mBeck", "mBeckett", "fBecky", "fBelinda", "fBella", "mBellamy", "fBellatrix", "fBelle", "mBen"
    , "mBenedict", "fBenita", "mBenjamin", "mBenji", "mBenjy", "mBennett", "mBennie", "mBenny", "mBenson", "mBentley"
    , "mBently", "fBernadette", "mBernard", "mBernardo", "fBernice", "mBernie", "mBert", "fBertha", "mBertie",
    "mBertram"
    , "fBeryl", "fBess", "fBeth", "fBethan", "fBethanie", "fBethany", "fBetsy", "fBettina", "fBetty", "mBev"
    , "mBevan", "fBeverly", "fBeyonce", "fBianca", "mBill", "fBillie", "mBilly", "mBjorn", "mBladen", "mBlain"
    , "mBlaine", "mBlair", "fBlair", "fBlaire", "mBlaise", "mBlake", "fBlake", "fBlakely", "fBlanche", "fBlaze"
    , "mBlaze", "fBlessing", "fBliss", "fBloom", "fBlossom", "mBlue", "fBlythe", "mBob", "fBobbi", "fBobbie"
    , "fBobby", "mBobby", "mBodie", "fBonita", "fBonnie", "fBonquesha", "mBoris", "mBoston", "mBowen", "mBoyd"
    , "mBrad", "mBraden", "mBradford", "mBradley", "mBradwin", "mBrady", "mBraeden", "fBraelyn", "mBram", "mBranden"
    , "fBrandi", "mBrandon", "fBrandy", "mBrantley", "mBraxton", "mBrayan", "mBrayden", "mBraydon", "fBraylee",
    "mBraylon"
    , "fBrea", "fBreanna", "fBree", "fBreeze", "fBrenda", "mBrendan", "mBrenden", "mBrendon", "fBrenna", "mBrennan"
    , "mBrent", "mBrenton", "mBret", "mBrett", "mBrevin", "mBrevyn", "fBria", "mBrian", "fBriana", "fBrianna"
    , "fBrianne", "fBriar", "mBrice", "fBridget", "fBridgette", "mBridie", "fBridie", "fBriella", "fBrielle",
    "mBrighton"
    , "fBrigid", "fBriley", "fBrinley", "mBrinley", "fBriony", "fBrisa", "fBristol", "fBritney", "fBritt", "fBrittany"
    , "fBrittney", "mBrock", "mBrodie", "mBrody", "mBrogan", "fBrogan", "fBronagh", "mBronson", "fBronte", "fBronwen"
    , "fBronwyn", "fBrook", "fBrooke", "fBrooklyn", "fBrooklynn", "mBrooks", "mBruce", "mBruno", "mBryan", "fBryanna"
    , "mBryant", "mBryce", "mBryden", "mBrydon", "fBrylee", "fBryn", "fBrynlee", "fBrynn", "mBryon", "fBryony"
    , "mBryson", "mBuck", "mBuddy", "fBunty", "mBurt", "mBurton", "mBuster", "mButch", "mByron", "mCadby"
    , "mCade", "mCaden", "fCadence", "mCael", "mCaelan", "mCaesar", "mCai", "mCaiden", "fCailin", "mCain"
    , "fCaitlan", "fCaitlin", "fCaitlyn", "mCaius", "mCal", "mCale", "mCaleb", "fCaleigh", "mCalhoun", "fCali"
    , "fCalista", "mCallan", "mCallen", "fCallie", "fCalliope", "fCallista", "mCallum", "mCalum", "mCalvin", "fCalypso"
    , "mCam", "fCambria", "mCamden", "mCameron", "fCameron", "fCami", "fCamila", "fCamilla", "fCamille", "mCampbell"
    , "mCamron", "fCamryn", "fCandace", "fCandice", "fCandis", "fCandy", "fCaoimhe", "fCaprice", "fCara", "mCarey"
    , "fCarina", "fCaris", "fCarissa", "mCarl", "fCarla", "fCarlene", "fCarley", "fCarlie", "mCarlisle", "mCarlos"
    , "mCarlton", "fCarly", "fCarlynn", "fCarmel", "fCarmela", "fCarmen", "fCarol", "fCarole", "fCarolina", "fCaroline"
    , "fCarolyn", "fCarrie", "mCarsen", "mCarson", "mCarter", "fCarter", "mCary", "fCarys", "fCasey", "mCasey"
    , "mCash", "mCason", "mCasper", "fCassandra", "fCassia", "fCassidy", "fCassie", "mCassius", "mCastiel", "mCastor"
    , "fCat", "fCatalina", "fCate", "fCaterina", "mCathal", "fCathalina", "fCatherine", "fCathleen", "fCathy", "fCatlin"
    , "mCato", "fCatrina", "fCatriona", "mCavan", "mCayden", "mCaydon", "fCayla", "fCece", "fCecelia", "mCecil"
    , "fCecilia", "fCecily", "mCedric", "fCeleste", "fCelestia", "fCelestine", "fCelia", "fCelina", "fCeline", "fCelise"
    , "fCerise", "fCerys", "mCesar", "mChad", "mChance", "mChandler", "fChanel", "fChanelle", "mChanning", "fChantal"
    , "fChantelle", "fCharis", "fCharissa", "fCharity", "fCharlene", "mCharles", "fCharley", "mCharley", "mCharlie",
    "fCharlie"
    , "fCharlize", "fCharlotte", "mCharlton", "fCharmaine", "mChase", "fChastity", "mChaz", "mChe", "fChelsea",
    "fChelsey"
    , "fChenai", "fChenille", "fCher", "fCheri", "fCherie", "fCherry", "fCheryl", "mChesney", "mChester", "mChevy"
    , "fCheyanne", "fCheyenne", "fChiara", "mChip", "fChloe", "mChris", "fChris", "fChrissy", "fChrista", "fChristabel"
    , "fChristal", "fChristen", "fChristi", "mChristian", "fChristiana", "fChristie", "fChristina", "fChristine",
    "mChristopher", "fChristy"
    , "fChrystal", "mChuck", "mCian", "fCiara", "mCiaran", "fCici", "fCiel", "fCierra", "mCillian", "fCindy"
    , "fClaire", "mClancy", "fClara", "fClarabelle", "fClare", "mClarence", "fClarice", "fClaris", "fClarissa",
    "fClarisse"
    , "fClarity", "mClark", "fClary", "mClaude", "fClaudette", "fClaudia", "fClaudine", "mClay", "mClayton", "fClea"
    , "mClement", "fClementine", "fCleo", "fCleopatra", "mCliff", "mClifford", "mClifton", "mClint", "mClinton",
    "mClive"
    , "fClodagh", "fClotilde", "fClover", "mClyde", "mCoby", "fCoco", "mCody", "mCohen", "mColby", "mCole"
    , "fColette", "mColin", "fColleen", "mCollin", "mColm", "mColt", "mColton", "mConan", "mConner", "fConnie"
    , "mConnor", "mConor", "mConrad", "fConstance", "mConstantine", "mCooper", "fCora", "fCoral", "fCoralie",
    "fCoraline"
    , "mCorbin", "fCordelia", "mCorey", "fCori", "fCorina", "fCorinne", "mCormac", "fCornelia", "mCornelius", "fCorra"
    , "mCory", "fCosette", "fCourtney", "mCraig", "fCressida", "fCristal", "mCristian", "fCristina", "mCristobal",
    "mCrosby"
    , "mCruz", "fCrystal", "mCullen", "mCurt", "mCurtis", "mCuthbert", "fCyndi", "fCynthia", "mCyril", "mCyrus"
    , "mDacey", "fDagmar", "fDahlia", "mDaire", "fDaisy", "fDakota", "mDakota", "mDale", "mDallas", "mDalton"
    , "mDamian", "mDamien", "mDamion", "mDamon", "mDan", "fDana", "mDana", "mDane", "fDanette", "fDani"
    , "fDanica", "mDaniel", "fDaniela", "fDaniella", "fDanielle", "fDanika", "mDanny", "mDante", "fDaphne", "mDara"
    , "fDara", "mDaragh", "fDarby", "fDarcey", "fDarcie", "mDarcy", "fDarcy", "mDaren", "fDaria", "mDarian"
    , "mDarin", "mDario", "mDarius", "fDarla", "fDarlene", "mDarnell", "mDarragh", "mDarrel", "mDarrell", "mDarren"
    , "mDarrin", "mDarryl", "mDarryn", "mDarwin", "mDaryl", "mDash", "mDashawn", "fDasia", "mDave", "mDavid"
    , "fDavida", "mDavin", "fDavina", "mDavion", "mDavis", "fDawn", "mDawson", "mDax", "mDaxter", "mDaxton"
    , "fDayna", "fDaysha", "mDayton", "mDeacon", "mDean", "fDeana", "fDeandra", "mDeandre", "fDeann", "fDeanna"
    , "fDeanne", "fDeb", "fDebbie", "fDebby", "fDebora", "fDeborah", "fDebra", "mDeclan", "fDee", "fDeedee"
    , "fDeena", "mDeepak", "fDeidre", "fDeirdre", "fDeja", "fDelaney", "fDelanie", "fDelany", "mDelbert", "fDelia"
    , "fDelilah", "fDella", "fDelores", "fDelphine", "fDemetria", "mDemetrius", "fDemi", "fDena", "mDenis", "fDenise"
    , "mDennis", "fDenny", "mDenver", "mDenzel", "mDeon", "mDerek", "mDermot", "mDerrick", "mDeshaun", "mDeshawn"
    , "fDesiree", "mDesmond", "fDestinee", "fDestiny", "mDev", "mDevin", "mDevlin", "mDevon", "mDewayne", "mDewey"
    , "mDexter", "fDiamond", "fDiana", "fDiane", "fDianna", "fDianne", "mDiarmuid", "mDick", "fDido", "mDiego"
    , "mDilan", "mDillon", "mDimitri", "fDina", "mDinesh", "mDino", "mDion", "fDionne", "fDior", "mDirk"
    , "fDixie", "mDjango", "mDmitri", "fDolly", "fDolores", "mDominic", "mDominick", "fDominique", "mDon", "mDonald"
    , "fDonna", "mDonnie", "mDonovan", "fDora", "fDoreen", "mDorian", "fDoris", "fDorothy", "fDot", "mDoug"
    , "mDouglas", "mDoyle", "mDrake", "mDrew", "fDrew", "mDuane", "mDuke", "fDulce", "mDuncan", "mDustin"
    , "mDwayne", "mDwight", "mDylan", "fEabha", "mEamon", "mEarl", "mEarnest", "mEason", "mEaston", "fEbony"
    , "fEcho", "mEd", "mEddie", "mEddy", "fEden", "mEden", "mEdgar", "fEdie", "mEdison", "fEdith"
    , "mEdmund", "fEdna", "mEdouard", "mEdric", "mEdsel", "mEduardo", "mEdward", "mEdwardo", "mEdwin", "fEdwina"
    , "fEffie", "mEfrain", "mEfren", "mEgan", "mEgon", "fEileen", "fEilidh", "fEimear", "fElaina", "fElaine"
    , "fElana", "fEleanor", "fElectra", "fElektra", "fElena", "mEli", "fEliana", "mElias", "mElijah", "fElin"
    , "fElina", "fElinor", "mEliot", "fElisa", "fElisabeth", "fElise", "mElisha", "fEliza", "fElizabeth", "fElla"
    , "fElle", "fEllen", "fEllery", "fEllie", "mEllington", "mElliot", "mElliott", "fEllis", "mEllis", "fElly"
    , "mElmer", "mElmo", "fElodie", "fEloise", "fElora", "fElsa", "fElsie", "fElspeth", "mElton", "fElva"
    , "fElvira", "mElvis", "mElwyn", "fElysia", "fElyza", "mEmanuel", "fEmanuela", "fEmber", "fEmelda", "fEmely"
    , "fEmer", "fEmerald", "mEmerson", "fEmerson", "mEmery", "mEmet", "mEmil", "fEmilee", "fEmilia", "mEmiliano"
    , "fEmilie", "mEmilio", "fEmily", "fEmma", "fEmmalee", "fEmmaline", "fEmmalyn", "mEmmanuel", "fEmmanuelle",
    "fEmmeline"
    , "mEmmerson", "mEmmet", "mEmmett", "fEmmie", "fEmmy", "fEnid", "mEnnio", "mEnoch", "mEnrique", "fEnya"
    , "mEnzo", "mEoghan", "mEoin", "mEric", "fErica", "mErick", "mErik", "fErika", "fErin", "fEris"
    , "mErnest", "mErnesto", "mErnie", "mErrol", "mErvin", "mErwin", "fEryn", "fEsmay", "fEsme", "fEsmeralda"
    , "fEsparanza", "fEsperanza", "mEsteban", "fEstee", "fEstelle", "fEster", "fEsther", "fEstrella", "mEthan", "fEthel"
    , "mEthen", "mEtienne", "mEuan", "mEuen", "mEugene", "fEugenie", "fEunice", "mEustace", "fEva", "mEvan"
    , "fEvangelina", "fEvangeline", "mEvangelos", "fEve", "fEvelin", "fEvelyn", "mEvelyn", "mEverett", "fEverly",
    "fEvie"
    , "fEvita", "mEwan", "mEzekiel", "mEzio", "mEzra", "mFabian", "mFabio", "fFabrizia", "mFaisal", "fFaith"
    , "fFallon", "fFanny", "fFarah", "mFarley", "fFarrah", "fFatima", "fFawn", "fFay", "fFaye", "mFebian"
    , "fFelicia", "fFelicity", "mFelipe", "mFelix", "mFergus", "fFern", "mFernand", "fFernanda", "mFernando", "fFfion"
    , "mFidel", "fFifi", "mFinbar", "mFinlay", "mFinley", "mFinn", "mFinnian", "mFinnigan", "fFiona", "mFionn"
    , "mFletcher", "fFleur", "fFlick", "fFlo", "fFlora", "fFlorence", "mFloyd", "mFlynn", "mFord", "mForest"
    , "mForrest", "mFoster", "mFox", "fFran", "fFrances", "fFrancesca", "mFrancesco", "fFrancine", "mFrancis",
    "mFrancisco"
    , "mFrank", "fFrankie", "mFrankie", "mFranklin", "mFranklyn", "mFraser", "mFred", "fFreda", "mFreddie", "mFreddy"
    , "mFrederick", "mFredrick", "fFreya", "fFrida", "mFritz", "fGabby", "mGabe", "mGabriel", "fGabriela", "fGabriella"
    , "fGabrielle", "mGael", "mGaelan", "mGage", "fGail", "mGale", "mGalen", "mGannon", "mGareth", "mGarman"
    , "fGarnet", "mGarrett", "mGarrison", "mGarry", "mGarth", "mGary", "mGaston", "mGavin", "fGayle", "fGaynor"
    , "fGeena", "fGemma", "fGena", "mGene", "fGenesis", "fGenevieve", "mGeoff", "mGeoffrey", "mGeorge", "fGeorgette"
    , "fGeorgia", "fGeorgie", "fGeorgina", "mGeraint", "mGerald", "fGeraldine", "mGerard", "mGerardo", "mGermain",
    "mGerry"
    , "fGert", "fGertrude", "fGia", "mGian", "fGianna", "mGibson", "mGideon", "fGigi", "mGil", "mGilbert"
    , "mGilberto", "mGiles", "fGillian", "fGina", "fGinger", "fGinny", "mGino", "mGiorgio", "fGiovanna", "mGiovanni"
    , "fGisela", "fGiselle", "fGisselle", "fGladys", "mGlen", "fGlenda", "mGlenn", "fGlenys", "fGloria", "mGlyndwr"
    , "fGlynis", "mGodfrey", "mGodric", "mGodwin", "fGolda", "fGoldie", "mGonzalo", "mGordon", "fGrace", "fGracelyn"
    , "fGracie", "mGrady", "mGraeme", "mGraham", "fGrainne", "mGrant", "mGrayson", "mGreg", "mGregg", "mGregor"
    , "mGregory", "fGreta", "fGretchen", "mGrey", "mGreyson", "mGriffin", "fGriselda", "fGuadalupe", "mGuillermo",
    "fGuinevere"
    , "mGunnar", "mGunner", "mGus", "mGustav", "mGustavo", "mGuy", "fGwen", "fGwendolyn", "fGwyneth", "fHabiba"
    , "mHaden", "fHadley", "mHaiden", "fHailee", "fHailey", "mHal", "fHaleigh", "fHaley", "fHalle", "fHallie"
    , "mHamish", "mHan", "mHank", "fHanna", "fHannah", "mHans", "mHarlan", "fHarley", "mHarley", "fHarmony"
    , "mHarold", "fHarper", "fHarriet", "mHarris", "mHarrison", "mHarry", "mHarvey", "mHassan", "fHattie", "fHaven"
    , "mHayden", "fHayden", "mHayes", "fHaylee", "fHayley", "fHazel", "fHazeline", "mHeath", "fHeather", "fHeaven"
    , "mHector", "fHeidi", "fHelen", "fHelena", "fHelene", "fHelga", "fHelina", "mHendrik", "mHendrix", "mHenley"
    , "mHenri", "fHenrietta", "mHenry", "fHepsiba", "fHera", "mHerbert", "mHerman", "fHermione", "fHester", "mHeston"
    , "fHetty", "fHilary", "mHilary", "fHilda", "fHillary", "mHolden", "fHollie", "fHolly", "mHomer", "fHonesty"
    , "fHoney", "fHonor", "fHonour", "fHope", "mHorace", "mHoratio", "mHoward", "mHubert", "mHudson", "mHugh"
    , "mHugo", "mHumberto", "mHumphrey", "mHunter", "mHuw", "fHyacinth", "mHywel", "mIain", "mIan", "fIanthe"
    , "mIanto", "mIbrahim", "fIda", "mIdris", "mIeuan", "mIggy", "mIgnacio", "mIgor", "mIke", "fIla"
    , "fIlene", "fIliana", "fIlona", "fIlse", "fImani", "fImelda", "fImogen", "mImran", "fIndia", "mIndiana"
    , "fIndie", "fIndigo", "fIndira", "fInes", "fIngrid", "mInigo", "fIona", "mIra", "fIra", "fIrene"
    , "fIrina", "fIris", "fIrma", "mIrvin", "mIrving", "mIrwin", "fIsa", "mIsaac", "fIsabel", "fIsabell"
    , "fIsabella", "fIsabelle", "fIsadora", "mIsaiah", "fIsha", "mIsiah", "mIsidore", "fIsis", "fIsla", "mIsmael"
    , "fIsobel", "fIsolde", "mIsrael", "mIssac", "fItzel", "mIvan", "fIvana", "mIvor", "fIvy", "fIyanna"
    , "fIzabella", "fIzidora", "fIzzie", "fIzzy", "mJace", "fJacinda", "fJacinta", "mJack", "mJackie", "fJackie"
    , "mJackson", "mJacob", "mJacoby", "fJacqueline", "fJacquelyn", "mJacques", "fJada", "fJade", "fJaden", "mJaden"
    , "mJadon", "fJadyn", "fJaelynn", "mJagger", "mJago", "mJai", "fJaida", "mJaiden", "fJaime", "mJaime"
    , "mJak", "mJake", "mJakob", "mJalen", "mJamal", "mJames", "mJameson", "mJamie", "fJamie", "mJamison"
    , "fJamiya", "fJan", "mJan", "fJana", "fJancis", "fJane", "fJanelle", "fJanessa", "fJanet", "fJanette"
    , "fJania", "fJanice", "fJanie", "fJanine", "fJanis", "fJaniya", "fJanuary", "fJaqueline", "mJared", "mJarod"
    , "mJarrett", "mJarrod", "mJarvis", "mJase", "fJasmin", "fJasmine", "mJason", "mJasper", "mJavier", "mJavon"
    , "mJax", "mJaxon", "mJaxson", "mJay", "fJaya", "mJayce", "fJayda", "mJayden", "fJayden", "mJaydon"
    , "fJayla", "mJaylen", "fJaylene", "mJaylin", "fJaylinn", "mJaylon", "fJaylynn", "fJayne", "mJayson", "fJazlyn"
    , "fJazmin", "fJazmine", "fJazz", "fJean", "fJeanette", "fJeanine", "fJeanne", "fJeannette", "fJeannie", "fJeannine"
    , "mJeb", "mJebediah", "mJed", "mJediah", "mJedidiah", "mJeff", "mJefferson", "mJeffery", "mJeffrey", "mJeffry"
    , "fJemima", "fJemma", "fJen", "fJena", "fJenelle", "fJenessa", "fJenna", "fJennette", "fJenni", "fJennie"
    , "fJennifer", "fJenny", "fJensen", "mJensen", "mJenson", "mJerald", "mJeremiah", "mJeremy", "fJeri", "mJericho"
    , "mJermaine", "mJerome", "fJerri", "mJerry", "fJess", "fJessa", "mJesse", "fJessica", "mJessie", "fJessie"
    , "mJesus", "fJet", "mJet", "mJethro", "mJett", "fJewel", "fJill", "fJillian", "mJim", "mJimmie"
    , "mJimmy", "fJo", "mJoachim", "fJoan", "fJoann", "fJoanna", "fJoanne", "mJoaquin", "fJocelyn", "fJodi"
    , "fJodie", "fJody", "mJody", "mJoe", "mJoel", "fJoelle", "mJoey", "mJohan", "fJohanna", "mJohn"
    , "mJohnathan", "mJohnathon", "mJohnnie", "mJohnny", "fJoleen", "fJolene", "fJolie", "mJon", "mJonah", "mJonas"
    , "mJonathan", "mJonathon", "fJoni", "mJonty", "mJordan", "fJordan", "fJordana", "mJordon", "mJordy", "fJordyn"
    , "mJorge", "fJorja", "mJose", "fJoselyn", "mJoseph", "fJosephine", "mJosh", "mJoshua", "mJosiah", "fJosie"
    , "mJosue", "mJovan", "fJoy", "fJoyce", "mJuan", "fJuanita", "mJudah", "mJudas", "mJudd", "fJude"
    , "mJude", "fJudith", "fJudy", "fJules", "fJulia", "mJulian", "fJuliana", "fJulianna", "fJulianne", "fJulie"
    , "fJulienne", "fJuliet", "fJuliette", "mJulio", "fJulissa", "mJulius", "fJuly", "fJune", "fJuniper", "fJuno"
    , "fJustice", "mJustice", "mJustin", "fJustina", "fJustine", "fKacey", "mKade", "mKaden", "fKadence", "mKai"
    , "mKaiden", "fKaidence", "fKailey", "fKailyn", "mKaine", "fKaitlin", "fKaitlyn", "fKaitlynn", "mKale", "fKalea"
    , "mKaleb", "fKaleigh", "fKali", "fKalia", "fKalista", "fKallie", "fKamala", "mKameron", "fKamryn", "mKane"
    , "fKara", "fKaren", "fKari", "fKarin", "fKarina", "fKarissa", "mKarl", "fKarla", "fKarlee", "fKarly"
    , "fKarolina", "mKarson", "fKaryn", "fKasey", "mKash", "mKasper", "fKassandra", "fKassidy", "fKassie", "fKat"
    , "fKatara", "fKatarina", "fKate", "fKatelyn", "fKatelynn", "fKaterina", "fKatharine", "fKatherine", "fKathleen",
    "fKathryn"
    , "fKathy", "fKatia", "fKatie", "fKatlyn", "fKatniss", "fKatrina", "fKaty", "fKatya", "fKay", "fKaya"
    , "mKayden", "fKaye", "fKayla", "fKaylee", "fKayleigh", "mKaylen", "fKayley", "fKaylie", "fKaylin", "mKayson"
    , "mKeanu", "fKeara", "mKeaton", "mKedrick", "mKeegan", "fKeeley", "fKeely", "mKeenan", "fKeira", "fKeisha"
    , "mKeith", "fKelis", "mKellan", "mKellen", "fKelley", "fKelli", "fKellie", "mKellin", "mKelly", "fKelly"
    , "fKelsey", "fKelsie", "mKelvin", "mKen", "fKendall", "mKendall", "fKendra", "mKendrick", "fKenna", "fKennedy"
    , "mKennedy", "mKenneth", "mKenny", "mKent", "mKenton", "fKenzie", "fKera", "fKeri", "fKerian", "fKerri"
    , "fKerry", "mKerry", "mKevin", "mKhalid", "mKhalil", "fKia", "mKian", "fKiana", "fKiara", "mKiefer"
    , "fKiera", "mKieran", "mKieron", "fKierra", "fKiersten", "fKiki", "fKiley", "mKillian", "fKim", "mKim"
    , "fKimberlee", "fKimberley", "fKimberly", "fKimbriella", "fKimmy", "mKingsley", "mKingston", "fKinley", "fKinsey",
    "fKinsley"
    , "mKip", "fKira", "mKiran", "mKirby", "mKirk", "fKirsten", "fKirstin", "fKirsty", "mKit", "fKitty"
    , "fKizzy", "mKlaus", "mKlay", "fKloe", "mKnox", "mKobe", "mKoby", "mKody", "mKolby", "fKora"
    , "fKori", "fKourtney", "mKris", "fKris", "mKrish", "fKrista", "fKristen", "fKristi", "mKristian", "fKristie"
    , "fKristin", "fKristina", "fKristine", "mKristoff", "mKristopher", "fKristy", "fKrystal", "mKurt", "mKurtis",
    "mKye"
    , "fKyla", "mKylar", "mKyle", "fKylee", "fKyleigh", "mKylen", "mKyler", "fKylie", "fKyra", "mKyran"
    , "mKyrin", "mKyron", "fLacey", "mLacey", "mLachlan", "fLacie", "fLacy", "fLadonna", "fLaila", "fLainey"
    , "mLake", "fLakyn", "fLala", "mLamar", "mLamont", "fLana", "mLance", "mLanden", "mLandon", "mLandyn"
    , "mLane", "fLaney", "mLangdon", "mLangston", "fLara", "fLarissa", "mLarry", "mLars", "fLatoya", "fLaura"
    , "fLaurel", "fLauren", "mLaurence", "fLaurie", "mLaurie", "fLauryn", "fLavana", "fLavender", "fLavinia",
    "mLawrence"
    , "mLawson", "fLayla", "mLayne", "mLayton", "fLea", "mLeaf", "fLeah", "fLeandra", "mLeandro", "fLeann"
    , "fLeanna", "fLeanne", "mLebron", "fLee", "mLee", "fLeela", "fLeena", "fLeia", "mLeigh", "fLeigh"
    , "mLeighton", "fLeila", "fLeilani", "fLela", "mLeland", "fLena", "mLennie", "mLennon", "mLennox", "mLenny"
    , "fLenore", "mLeo", "mLeon", "fLeona", "mLeonard", "mLeonardo", "mLeonel", "fLeonie", "mLeopold", "fLeora"
    , "mLeroy", "mLes", "fLesley", "mLeslie", "fLeslie", "fLesly", "mLester", "fLeticia", "fLetitia", "fLettie"
    , "mLeuan", "mLev", "mLeven", "mLevi", "mLewis", "mLex", "fLexi", "fLexia", "fLexie", "fLexis"
    , "fLeyla", "fLia", "mLiam", "fLiana", "fLianne", "fLibbie", "fLibby", "fLiberty", "fLidia", "mLief"
    , "fLiesl", "fLila", "fLilac", "fLilah", "fLili", "fLilian", "fLiliana", "fLilita", "fLilith", "fLillia"
    , "fLillian", "fLillie", "fLilly", "fLily", "fLina", "mLincoln", "fLinda", "fLindsay", "fLindsey", "fLindy"
    , "mLink", "mLinus", "mLionel", "fLisa", "mLisandro", "fLisette", "fLiv", "fLivia", "fLivvy", "fLiz"
    , "fLiza", "fLizbeth", "fLizette", "fLizzie", "fLizzy", "mLloyd", "mLochlan", "mLogan", "fLogan", "fLois"
    , "mLoki", "fLola", "fLolita", "fLondon", "mLondon", "mLonnie", "fLora", "fLoran", "mLorcan", "fLorelei"
    , "mLoren", "fLoren", "fLorena", "mLorenzo", "fLoretta", "fLori", "fLorie", "mLoris", "fLorna", "fLorraine"
    , "fLorri", "fLorrie", "fLottie", "fLotus", "fLou", "mLou", "fLouella", "mLouie", "mLouis", "fLouisa"
    , "fLouise", "mLowell", "fLuann", "mLuca", "mLucas", "fLucia", "mLucian", "fLuciana", "mLuciano", "fLucie"
    , "fLucille", "fLucinda", "fLucky", "fLucy", "mLuigi", "mLuis", "fLuisa", "mLukas", "mLuke", "fLulu"
    , "fLuna", "fLupita", "mLuther", "fLuz", "fLydia", "fLyla", "mLyle", "fLynda", "mLyndon", "fLyndsey"
    , "fLynette", "mLynn", "fLynn", "fLynne", "fLynnette", "fLynsey", "fLyra", "fLyric", "mLysander", "fMabel"
    , "fMacey", "fMacie", "mMack", "fMackenzie", "fMacy", "fMadalyn", "fMaddie", "fMaddison", "mMaddox", "fMaddy"
    , "fMadeleine", "fMadeline", "fMadelyn", "fMadison", "fMadisyn", "fMadonna", "fMadyson", "fMae", "fMaeve", "fMagda"
    , "fMagdalena", "fMagdalene", "fMaggie", "mMagnus", "fMaia", "fMaire", "fMairead", "fMaisie", "mMaison", "fMaisy"
    , "fMaja", "fMakayla", "fMakenna", "fMakenzie", "mMalachi", "mMalakai", "mMalcolm", "fMalia", "mMalik", "fMalina"
    , "fMalinda", "fMallory", "mMalloy", "fMalory", "fMandy", "mManny", "mManuel", "fManuela", "fMara", "mMarc"
    , "mMarcel", "fMarcela", "fMarcella", "fMarcelle", "fMarci", "fMarcia", "fMarcie", "mMarco", "mMarcos", "mMarcus"
    , "fMarcy", "fMargaret", "fMargarita", "fMargaux", "fMarge", "fMargie", "fMargo", "fMargot", "fMargret", "fMaria"
    , "fMariah", "fMariam", "fMarian", "fMariana", "fMarianna", "fMarianne", "fMaribel", "fMarie", "fMariela",
    "fMariella"
    , "mMarik", "fMarilyn", "fMarina", "mMario", "mMarion", "fMarion", "fMarisa", "fMarisol", "fMarissa", "fMaritza"
    , "fMarjorie", "mMark", "fMarla", "fMarlee", "fMarlena", "fMarlene", "mMarley", "fMarley", "mMarlon", "fMarnie"
    , "mMarquis", "fMarsha", "mMarshall", "fMartha", "mMartin", "fMartina", "mMarty", "mMartyn", "mMarvin", "fMary"
    , "fMaryam", "fMaryann", "fMarybeth", "fMasie", "mMason", "mMassimo", "mMat", "mMateo", "mMathew", "fMatilda"
    , "mMatt", "mMatthew", "mMatthias", "fMaude", "fMaura", "fMaureen", "mMaurice", "mMauricio", "mMaverick", "fMavis"
    , "mMax", "mMaxim", "mMaximilian", "mMaximus", "fMaxine", "mMaxwell", "fMay", "fMaya", "fMazie", "fMckayla"
    , "fMckenna", "fMckenzie", "fMea", "fMeadow", "fMeagan", "fMeera", "fMeg", "fMegan", "fMeghan", "mMehdi"
    , "mMehtab", "fMei", "mMekhi", "mMel", "fMel", "fMelanie", "fMelina", "fMelinda", "fMelissa", "fMelody"
    , "mMelvin", "fMercedes", "fMercy", "fMeredith", "mMerick", "fMerida", "mMervyn", "fMeryl", "fMia", "mMicah"
    , "mMichael", "fMichaela", "mMicheal", "fMichele", "fMichelle", "mMick", "mMickey", "mMiguel", "fMika", "fMikaela"
    , "fMikayla", "mMike", "mMikey", "fMikhaela", "fMila", "mMilan", "fMildred", "fMilena", "mMiles", "fMiley"
    , "mMiller", "fMillicent", "fMillie", "fMilly", "mMilo", "mMilton", "fMimi", "fMina", "fMindy", "fMinerva"
    , "fMinnie", "fMira", "fMirabel", "fMirabelle", "fMiracle", "fMiranda", "fMiriam", "fMirielle", "mMisha", "fMissie"
    , "fMisty", "mMitch", "mMitchell", "mMitt", "fMitzi", "mMoe", "mMohamed", "mMohammad", "mMohammed", "fMoira"
    , "mMoises", "fMollie", "fMolly", "fMona", "fMonica", "fMonika", "fMonique", "fMontana", "mMonte", "fMontserrat"
    , "mMonty", "mMordecai", "mMorgan", "fMorgan", "fMorgana", "mMorris", "mMoses", "fMoya", "mMuhammad", "fMuriel"
    , "mMurphy", "mMurray", "fMya", "fMyfanwy", "fMyla", "mMyles", "fMyra", "fMyrna", "mMyron", "fMyrtle"
    , "fNadene", "fNadia", "fNadine", "fNaja", "fNala", "fNana", "fNancy", "fNanette", "fNaomi", "mNash"
    , "mNasir", "fNatalia", "fNatalie", "fNatasha", "mNate", "mNath", "mNathan", "mNathanael", "mNathaniel", "fNaya"
    , "fNayeli", "mNeal", "mNed", "mNehemiah", "mNeil", "fNell", "fNellie", "fNelly", "mNelson", "fNena"
    , "fNerissa", "mNesbit", "fNessa", "mNestor", "fNevaeh", "fNeve", "mNeville", "mNevin", "fNia", "mNiall"
    , "fNiamh", "fNichola", "mNicholas", "fNichole", "mNick", "fNicki", "mNickolas", "fNicky", "mNicky", "mNico"
    , "fNicola", "mNicolas", "fNicole", "fNicolette", "fNieve", "mNigel", "fNiki", "fNikita", "fNikki", "mNiklaus"
    , "mNikolai", "mNikolas", "fNila", "mNile", "mNils", "fNina", "fNishka", "mNoah", "mNoe", "mNoel"
    , "fNoelle", "fNoemi", "fNola", "mNolan", "fNora", "fNorah", "mNorbert", "fNoreen", "fNorma", "mNorman"
    , "fNova", "fNyla", "mOakes", "mOakley", "fOasis", "fOcean", "fOctavia", "mOctavio", "fOdalis", "fOdalys"
    , "fOdele", "fOdelia", "fOdette", "mOisin", "mOlaf", "fOlga", "mOli", "fOlive", "mOliver", "fOlivia"
    , "mOllie", "mOlly", "mOmar", "fOona", "fOonagh", "fOpal", "fOphelia", "fOprah", "mOran", "fOriana"
    , "fOrianna", "mOrion", "fOrla", "fOrlaith", "mOrlando", "mOrson", "mOscar", "mOsvaldo", "mOswald", "mOtis"
    , "mOtto", "mOwen", "mOzzie", "mOzzy", "mPablo", "mPaco", "mPaddy", "mPadraig", "fPage", "fPaige"
    , "fPaisley", "mPalmer", "fPaloma", "fPam", "fPamela", "fPandora", "fPansy", "fPaola", "mPaolo", "fParis"
    , "mParker", "mPascal", "mPat", "fPatience", "fPatrice", "fPatricia", "mPatrick", "fPatsy", "fPatti", "fPatty"
    , "mPaul", "fPaula", "fPaulette", "fPaulina", "fPauline", "mPaxton", "fPayton", "mPayton", "fPeace", "mPearce"
    , "fPearl", "mPedro", "fPeggy", "fPenelope", "fPenny", "mPercy", "fPerla", "fPerrie", "mPerry", "fPersephone"
    , "mPetar", "mPete", "mPeter", "fPetra", "fPetunia", "fPeyton", "mPeyton", "mPhebian", "mPhil", "mPhilip"
    , "mPhilippe", "mPhillip", "fPhillipa", "fPhilomena", "mPhineas", "fPhoebe", "fPhoenix", "mPhoenix", "fPhyllis",
    "mPierce"
    , "mPiers", "mPip", "fPiper", "fPippa", "fPixie", "fPolly", "fPollyanna", "fPoppy", "mPorter", "fPortia"
    , "mPoul", "mPrakash", "fPrecious", "fPresley", "fPreslie", "mPreston", "fPrimrose", "mPrince", "fPrincess",
    "mPrinceton"
    , "fPriscilla", "fPriya", "fPromise", "fPrudence", "fPrue", "fQueenie", "mQuentin", "fQuiana", "mQuincy", "mQuinlan"
    , "fQuinn", "mQuinn", "mQuinton", "mQuintrell", "fRabia", "fRachael", "fRachel", "fRachelle", "fRae", "fRaegan"
    , "fRaelyn", "mRafael", "mRafferty", "mRaheem", "mRahul", "mRaiden", "fRaina", "fRaine", "mRaj", "mRajesh"
    , "mRalph", "mRam", "mRameel", "mRamon", "fRamona", "mRamsey", "fRamsha", "mRandal", "mRandall", "fRandi"
    , "mRandolph", "mRandy", "fRani", "fRania", "mRaoul", "mRaphael", "fRaquel", "mRashad", "mRashan", "mRashid"
    , "mRaul", "fRaven", "mRavi", "mRay", "fRaya", "mRaylan", "mRaymond", "fRayna", "fRayne", "fReagan"
    , "fReanna", "fReanne", "fRebecca", "fRebekah", "mReece", "mReed", "mReef", "fReese", "mReese", "fRegan"
    , "mReggie", "fRegina", "mReginald", "mRehan", "mReid", "mReilly", "fReilly", "fReina", "mRemco", "fRemi"
    , "mRemington", "mRemy", "mRen", "fRena", "fRenata", "mRene", "fRene", "fRenee", "fRenesmee", "mReuben"
    , "mRex", "fReyna", "mReynaldo", "mReza", "fRhea", "mRhett", "fRhian", "fRhianna", "fRhiannon", "fRhoda"
    , "fRhona", "fRhonda", "mRhys", "fRia", "mRian", "fRianna", "mRicardo", "mRich", "mRichard", "mRichie"
    , "mRick", "mRickey", "fRicki", "mRickie", "mRicky", "mRico", "mRider", "fRihanna", "mRik", "mRiker"
    , "fRikki", "mRiley", "fRiley", "mRio", "fRita", "mRiver", "fRiver", "fRiya", "mRoan", "fRoanne"
    , "mRob", "mRobbie", "mRobby", "mRobert", "fRoberta", "mRoberto", "fRobin", "mRobin", "fRobyn", "mRocco"
    , "fRochelle", "fRocio", "mRock", "mRocky", "mRod", "mRoderick", "mRodger", "mRodney", "mRodolfo", "mRodrigo"
    , "mRogelio", "mRoger", "mRohan", "fRoisin", "mRoland", "fRolanda", "mRolando", "mRoman", "mRomeo", "mRon"
    , "mRonald", "mRonan", "fRonda", "fRoni", "mRonnie", "mRonny", "mRoosevelt", "mRory", "fRosa", "fRosalie"
    , "fRosalina", "fRosalind", "fRosalinda", "fRosalynn", "fRosanna", "mRoscoe", "fRose", "fRoseanne", "fRosella",
    "fRosemarie"
    , "fRosemary", "fRosetta", "fRosie", "mRoss", "fRosy", "fRowan", "mRowan", "fRowena", "fRoxana", "fRoxanne"
    , "fRoxie", "fRoxy", "mRoy", "mRoyce", "fRozlynn", "mRuairi", "mRuben", "mRubin", "fRuby", "mRudolph"
    , "mRudy", "fRue", "mRufus", "mRupert", "mRuss", "mRussell", "mRusty", "fRuth", "fRuthie", "mRyan"
    , "fRyanne", "fRydel", "mRyder", "mRyker", "mRylan", "mRyland", "fRylee", "fRyleigh", "mRyley", "fRylie"
    , "fSabina", "fSabine", "fSable", "fSabrina", "mSacha", "fSade", "fSadhbh", "fSadie", "fSaffron", "fSafire"
    , "fSafiya", "fSage", "fSahara", "mSaid", "fSaige", "fSaira", "fSally", "fSalma", "fSalome", "mSalvador"
    , "mSalvatore", "mSam", "fSam", "fSamantha", "fSamara", "fSamia", "mSamir", "fSamira", "fSammie", "fSammy"
    , "mSammy", "mSamson", "mSamuel", "mSandeep", "fSandra", "fSandy", "mSandy", "fSania", "mSanjay", "mSantiago"
    , "fSaoirse", "fSapphire", "fSara", "fSarah", "fSarina", "fSariya", "fSascha", "fSasha", "mSasha", "fSaskia"
    , "mSaul", "fSavanna", "fSavannah", "mSawyer", "fScarlet", "fScarlett", "mScot", "mScott", "mScottie", "mScotty"
    , "mSeamus", "mSean", "mSeb", "mSebastian", "fSebastianne", "mSebastien", "mSebestian", "fSelah", "fSelena",
    "fSelene"
    , "fSelina", "fSelma", "fSenuri", "fSeptember", "fSeren", "fSerena", "fSerenity", "mSergio", "mSeth", "mShadrach"
    , "fShakira", "fShana", "mShane", "fShania", "mShannon", "fShannon", "fShari", "fSharon", "fShary", "mShaun"
    , "fShauna", "mShawn", "fShawn", "fShawna", "fShawnette", "mShay", "fShayla", "fShayna", "mShayne", "fShea"
    , "mShea", "fSheba", "fSheena", "fSheila", "fShelby", "mSheldon", "fShelia", "fShelley", "fShelly", "mShelton"
    , "fSheri", "fSheridan", "mSherlock", "mSherman", "fSherri", "fSherrie", "fSherry", "fSheryl", "mShiloh", "fShirley"
    , "fShivani", "fShona", "fShonagh", "fShreya", "fShyla", "fSian", "mSid", "fSidney", "mSidney", "fSienna"
    , "fSierra", "fSigourney", "mSilas", "fSilvia", "mSimeon", "mSimon", "fSimone", "fSimran", "fSinead", "fSiobhan"
    , "fSky", "mSky", "fSkye", "mSkylar", "fSkylar", "mSkyler", "fSkyler", "mSlade", "fSloane", "fSnow"
    , "fSofia", "fSofie", "mSol", "mSolomon", "fSondra", "fSonia", "fSonja", "mSonny", "fSonya", "fSophia"
    , "fSophie", "fSophy", "mSoren", "fSorrel", "mSpencer", "mSpike", "fSpring", "mStacey", "fStacey", "fStaci"
    , "fStacie", "mStacy", "fStacy", "mStan", "mStanley", "fStar", "fStarla", "mStefan", "fStefanie", "fStella"
    , "fSteph", "mStephan", "fStephanie", "mStephen", "mSterling", "mSteve", "mSteven", "mStevie", "mStewart", "mStone"
    , "mStorm", "mStuart", "fSue", "mSufyan", "fSugar", "fSuki", "mSullivan", "fSummer", "fSusan", "fSusanna"
    , "fSusannah", "fSusanne", "fSusie", "fSutton", "fSuzanna", "fSuzanne", "fSuzette", "fSuzie", "fSuzy", "mSven"
    , "fSybil", "fSydney", "mSylvester", "fSylvia", "fSylvie", "fTabatha", "fTabitha", "mTadhg", "fTahlia", "fTala"
    , "fTalia", "fTalitha", "fTaliyah", "fTallulah", "mTalon", "mTam", "fTamara", "fTamera", "fTami", "fTamia"
    , "fTamika", "fTammi", "fTammie", "fTammy", "fTamra", "fTamsin", "fTania", "fTanika", "fTanisha", "mTanner"
    , "fTanya", "fTara", "mTariq", "mTarquin", "fTaryn", "fTasha", "fTasmin", "mTate", "fTatiana", "fTatum"
    , "fTawana", "fTaya", "fTayah", "fTayla", "fTaylah", "fTayler", "mTaylor", "fTaylor", "fTeagan", "mTed"
    , "mTeddy", "fTeegan", "fTegan", "fTeigan", "fTenille", "mTeo", "mTerence", "fTeresa", "fTeri", "mTerrance"
    , "mTerrell", "mTerrence", "fTerri", "fTerrie", "fTerry", "mTerry", "fTess", "fTessa", "mTevin", "mTex"
    , "mThad", "mThaddeus", "fThalia", "fThea", "fThelma", "mTheo", "fTheodora", "mTheodore", "mTheophilus", "fTheresa"
    , "fTherese", "mThomas", "fThomasina", "mThor", "fTia", "mTiago", "fTiana", "mTiberius", "fTiegan", "fTiffany"
    , "mTiger", "fTilly", "mTim", "mTimmy", "mTimothy", "fTina", "fTisha", "mTito", "mTitus", "mTobias"
    , "mTobin", "mToby", "mTod", "mTodd", "mTom", "mTomas", "mTommie", "mTommy", "fToni", "fTonia"
    , "mTony", "fTonya", "fTori", "mTorin", "mToryn", "mTrace", "fTracey", "mTracey", "fTraci", "fTracie"
    , "mTracy", "fTracy", "mTravis", "mTray", "mTremaine", "mTrent", "mTrenton", "mTrevon", "mTrevor", "mTrey"
    , "fTricia", "fTrina", "fTrinity", "fTrish", "fTrisha", "fTrista", "mTristan", "mTristen", "mTriston", "fTrixie"
    , "fTrixy", "mTroy", "fTrudy", "mTruman", "mTucker", "fTula", "fTulip", "mTy", "mTyler", "fTyra"
    , "mTyrese", "mTyrone", "mTyson", "fUlrica", "mUlysses", "fUma", "mUmar", "fUna", "mUriah", "mUriel"
    , "fUrsula", "mUsama", "mValentin", "fValentina", "mValentine", "mValentino", "fValeria", "fValerie", "fValery",
    "mVan"
    , "mVance", "fVanessa", "mVasco", "mVaughn", "fVeda", "fVelma", "fVenetia", "fVenus", "fVera", "fVerity"
    , "mVernon", "fVeronica", "fVicki", "fVickie", "fVicky", "mVictor", "fVictoria", "fVienna", "mVihan", "mVijay"
    , "mVikram", "mVince", "mVincent", "mVinnie", "fViola", "fViolet", "fVioletta", "mVirgil", "fVirginia", "mVishal"
    , "fVivian", "mVivian", "fViviana", "fVivien", "fVivienne", "mVlad", "mVladimir", "mWade", "mWalker", "mWallace"
    , "fWallis", "mWalter", "fWanda", "mWarren", "fWaverley", "mWaylon", "mWayne", "mWendell", "fWendi", "fWendy"
    , "mWes", "mWesley", "mWeston", "fWhitney", "mWilbert", "mWilbur", "mWiley", "mWilfred", "mWilhelm", "fWilhelmina"
    , "mWill", "fWilla", "mWillam", "mWillard", "mWillem", "mWilliam", "mWillie", "mWillis", "fWillow", "fWilma"
    , "mWilson", "fWinnie", "fWinnifred", "fWinona", "mWinston", "fWinter", "mWolfgang", "mWoody", "mWyatt", "mXander"
    , "fXandra", "fXanthe", "mXavier", "fXaviera", "fXena", "mXerxes", "fXia", "fXimena", "fXochil", "fXochitl"
    , "mYahir", "mYardley", "fYasmin", "fYasmine", "fYazmin", "mYehudi", "fYelena", "fYesenia", "mYestin", "fYolanda"
    , "mYork", "fYsabel", "fYulissa", "mYuri", "mYusuf", "fYvaine", "mYves", "fYvette", "fYvonne", "mZac"
    , "mZach", "mZachariah", "mZachary", "mZachery", "mZack", "mZackary", "mZackery", "fZada", "fZaheera", "fZahra"
    , "mZaiden", "mZain", "mZaine", "fZaira", "mZak", "fZakia", "fZali", "mZander", "mZane", "fZara"
    , "fZaria", "fZaya", "mZayden", "fZayla", "mZayn", "mZayne", "mZeb", "mZebulon", "mZed", "mZeke"
    , "fZelda", "fZelida", "fZelina", "fZena", "fZendaya", "mZeph", "fZia", "mZiggy", "fZina", "mZion"
    , "fZiva", "fZoe", "fZoey", "fZola", "mZoltan", "fZora", "fZoya", "fZula", "fZuri", "mZuriel"
    , "fZyana", "mZylen",
]


def gen_two_words(split=' ', lowercase=False):
    size = len(ALL_ENG_NAMES) - 1
    idx_first = random.randint(0, size)
    idx_last = random.randint(0, size)
    first_words = ALL_ENG_NAMES[idx_first][1:]
    last_words = ALL_ENG_NAMES[idx_last][1:]
    if lowercase:
        first_words = first_words.lower()
    last_words = last_words.lower()
    return '%s%s%s' % (first_words, split, last_words)


def gen_one_word_digit(lowercase=False, digitmax=1000):
    size = len(ALL_ENG_NAMES) - 1
    idx_name = random.randint(0, size)
    digit = random.randint(0, int(digitmax))
    name = ALL_ENG_NAMES[idx_name][1:]
    if lowercase:
        name = name.lower()
    return name + str(digit)


def gen_one_gender_word(male=False):
    size = len(ALL_ENG_NAMES) - 1
    while True:
        idx_name = random.randint(0, size)
    if male and ALL_ENG_NAMES[idx_name][0] == 'm':
        return ALL_ENG_NAMES[idx_name][1:]
    if not male and ALL_ENG_NAMES[idx_name][0] == 'f':
        return ALL_ENG_NAMES[idx_name][1:]


def gen_year(startyear, endyear):
    return str(random.randint(int(startyear), int(endyear)))


def gen_birthday(westenstyle=False):
    month = random.randint(1, 12)
    day = random.randint(1, 30)
    if not westenstyle:
        return str("%02d%02d" % (month, day))
    return str("%02d%02d" % (day, month))


if __name__ == '__main__':
    print
    gen_two_words(split=' ', lowercase=False)
    print
    gen_one_word_digit(lowercase=False)
    print
    gen_year(1988, 2015)
    print
    gen_birthday(westenstyle=True)
    print
    gen_two_words(split='', lowercase=True) + gen_year(1988, 2015)
    print
    gen_one_gender_word(male=True)



# encoding: utf-8
"""
@project: djangoModel->config
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/8/4 19:24
"""


# 用户令牌的密钥配置
JWT_AUTH = {
    'JWT_SECRET_KEY': '@zxmxy2021!',
    # 'JWT_EXPIRATION_DELTA': datetime.timedelta(days=365),
    # 'JWT_AUTH_HEADER_PREFIX': 'Bearer',
    # 'JWT_ALLOW_REFRESH': True,
}

# 用户令牌的过期时间配置
EXPIRE_TIME = {
    "DAY": 7,
    "SECOND": 0,
}


"""
Created on 2022-05-20
@author:刘飞
@description:用户验证装饰器，为单个请求方式做验证
"""
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from django.http import JsonResponse
from rest_framework import exceptions
from rest_framework.request import Request

from ..services.user_service import UserService


def user_authentication_wrapper(func):
    """
    用户认证装饰器，如果有Authorization则检查用户的有效性，如没有则认为用户以游客访问
    """

    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param request APIView实例会传入请求包
        @param request APIView实例会传入请求包
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        # print(instance, arg_request, request, args, kwargs)
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance * args, request=request, user_info={}, **kwargs, )
        # token = request.META.get('HTTP_AUTHORIZATION', None)
        token = request.headers.get('Authorization', None)
        # print("> user_authentication_wrapper token:", token, type(token))

        # 如果没有传token则可视为以游客身份访问
        if not token or str(token) == 'null' or str(token).strip().upper() == "BEARER":
            return func(instance, request=request, user_info=None, *args, **kwargs)

        user_serv, error_text = UserService.check_token(token)
        # print("> user_authentication_wrapper user_serv, error_text:", user_serv, error_text)
        if error_text:
            raise exceptions.AuthenticationFailed(error_text)
            # raise exceptions.AuthenticationFailed(error_text)

        request.user = user_serv
        result = func(instance, *args, request=request, user_info=user_serv, **kwargs)
        return result

    return wrapper


def user_authentication_force_wrapper(func):
    """
    用户认证装饰器，如果有Authorization则检查用户的有效性，如没有则认为用户以游客访问
    """

    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param request APIView实例会传入请求包
        @param request APIView实例会传入请求包
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance * args, request=request, user_info={}, **kwargs, )
        # token = request.META.get('HTTP_AUTHORIZATION', None) # 请求头不能将HTTP 及 _ 携带进请求头中，这样会导致系统识别失败
        token = request.headers.get('Authorization', None)

        # 如果没有传token则可视为以游客身份访问
        if not token or str(token) == 'null' or str(token).strip().upper() == "BEARER":
            return JsonResponse({'err': 6000, 'msg': '该用户未登录'})

        user_serv, error_text = UserService.check_token(token)
        if error_text:
            return JsonResponse({'err': 6001, 'msg': error_text})

        request.user = user_serv
        result = func(instance, *args, request=request, user_info=user_serv, **kwargs)
        return result

    return wrapper



# encoding: utf-8
"""
@project: djangoModel->user_group
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户分组接口
@created_time: 2022/7/22 14:10
"""
from rest_framework.views import APIView

from ..models import Group
from ..utils.model_handle import *
from ..validator import GroupValidator


class GroupAPIView(APIView):
    def list(self, *args):
        return util_response(data=list(Group.objects.all().values()))

    def post(self, request, *args):
        return model_create(request, Group, GroupValidator)

    def put(self, request, *args):
        return model_update(request, Group)

    def delete(self, request, *args):
        return model_delete(request, Group)



# encoding: utf-8
"""
@project: djangoModel->Auth
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 小程序SDK
@created_time: 2022/7/7 9:38
"""
from datetime import datetime, timedelta
import json
from logging import getLogger
from pathlib import Path

from django.contrib.auth.hashers import make_password
import jwt
import redis
import requests

from main.settings import BASE_DIR
from .user_detail_info_service import DetailInfoService
from ..models import BaseInfo, Auth, UserSsoToUser
from ..services.user_relate_service import UserRelateToUserService
from ..utils.custom_tool import get_short_id
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.model_handle import parse_model
from ..utils.nickname_generate import gen_one_word_digit

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_user"))

payment_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
payment_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

sub_appid = payment_main_config_dict.wechat_merchant_app_id or payment_module_config_dict.wechat_merchant_app_id or ""
sub_app_secret = payment_main_config_dict.wechat_merchant_app_secret or payment_module_config_dict.wechat_merchant_app_secret or ""

app_id = main_config_dict.app_id or module_config_dict.app_id or ""
app_secret = main_config_dict.secret or module_config_dict.secret or ""

jwt_secret_key = main_config_dict.jwt_secret_key or module_config_dict.jwt_secret_key or ""
expire_day = main_config_dict.expire_day or module_config_dict.expire_day or ""
expire_second = main_config_dict.expire_second or module_config_dict.expire_second or ""

redis_main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))
redis_module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="main"))

redis_host = redis_main_config_dict.redis_host or redis_module_config_dict.redis_host or ""
redis_port = redis_main_config_dict.redis_port or redis_module_config_dict.redis_port or ""
redis_password = redis_main_config_dict.redis_password or redis_module_config_dict.redis_password or ""

# print(">", sub_appid)
logger = getLogger('log')


class WechatService:
    wx_login_url = "https://api.weixin.qq.com/sns/jscode2session"
    wx_token_url = 'https://api.weixin.qq.com/cgi-bin/token'
    wx_get_phone_url = "https://api.weixin.qq.com/wxa/business/getuserphonenumber"

    def __init__(self):
        self.login_param = {'appid': app_id, 'secret': app_secret, 'grant_type': 'authorization_code'}
        self.redis = redis.Redis(
            host=redis_host,
            port=redis_port,
            password=redis_password
        )

    def get_openid(self, code):
        """
        :param code（openid登录的code）:
        :return:(err,data)
        """
        req_params = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'js_code': code,
            'grant_type': 'authorization_code',
        }
        user_info = requests.get('https://api.weixin.qq.com/sns/jscode2session', params=req_params, timeout=3,
                                 verify=False)
        return user_info.json()
        # try:
        #     response = requests.get(self.wx_login_url, code).json()
        #     if not response['errcode'] == 0:  # openid 换取失败
        #         return response['errcode'], response['errmsg']
        # except Exception as e:
        #     return 6445, '请求错误'

    def wechat_login(self, phone_code, login_code, sso_serve_id=None, detail_params=None):
        """
        过期续约就是重新登录
        :param detail_params:
        :param code: 换取手机号码的code
        :return:(err,data)
        """
        # code 换取手机号
        phone = ""
        try:
            if detail_params is None:
                detail_params = {}
            url = self.wx_get_phone_url + "?access_token={}".format(self.get_access_token()['access_token'])
            header = {'content-type': 'application/json'}
            response = requests.post(url, json={'code': phone_code}, headers=header).json()
            if not response['errcode'] == 0:
                return response['errmsg'], ""
            phone = response['phone_info']['phoneNumber']
            # 检查是否存在该用户，不存在直接注册
            # current_user = BaseInfo.objects.filter(phone=phone).filter()
            # current_user = parse_model(current_user)
            # if not current_user:
            #     base_info = {
            #         'user_name': '',
            #         'phone': phone,
            #         'email': '',
            #         'full_name': '请修改用户名',
            #     }
            #     current_user = BaseInfo.objects.create(**base_info)
            #     current_user = parse_model(current_user)
            # current_user = current_user[0]
            # # 生成登录token
            # token = self.__set_token(current_user.get('id', None), phone)
            # # 创建用户登录信息，绑定token
            # auth = {
            #     'user_id': current_user.get('id', None),
            #     'password': make_password('123456', None, 'pbkdf2_sha1'),
            #     'plaintext': '123456',
            #     'token': token,
            # }
            # Auth.objects.update_or_create({'user_id': current_user.get('id', None)}, **auth)
            # return 0, {'token': token, 'user_info': current_user}

            # 通过换取的手机号判断用户是否存在
            current_user = BaseInfo.objects.filter(phone=phone).filter()
            current_user = parse_model(current_user)
            if not login_code:
                return None, "微信登录 login_code 必传"
            wechat_openid = self.get_openid(code=login_code)
            if wechat_openid.get("openid", None) is None:
                return None, "获取 openid 失败 查openid是否过期, wechat_openid:" + json.dumps(wechat_openid)
            # 登录即注册操作
            if not current_user:
                # 用户不存在的时候，进行注册用户
                base_info = {
                    'user_name': get_short_id(8),  # 第一次注册的时候给一个唯一的字符串作登录账号
                    'nickname': gen_one_word_digit(),
                    'phone': phone,
                    'email': '',
                    'full_name': '请修改用户名',
                    'wechat_openid': wechat_openid.get("openid", None)
                }
                BaseInfo.objects.create(**base_info)
                current_user = BaseInfo.objects.filter(phone=phone).filter()
                current_user = parse_model(current_user)
                current_user = current_user[0]
                # 生成登录token
                token = self.__set_token(current_user.get('id', None), phone)
                # 用户第一次登录即注册，允许添加用户的详细信息
                try:
                    detail_params.setdefault("user_id", current_user.get('id', None))
                    data, detail_err = DetailInfoService.create_or_update_detail(detail_params)
                    if detail_err:
                        raise Exception(detail_err)
                except Exception as e:
                    logger.error('---首次登录写入用户详细信息异常：' + str(e) + '---')

                # 创建单点登录
                if sso_serve_id:
                    sso_data = {
                        "sso_serve_id": sso_serve_id,
                        "user_id": current_user.get('id', None),
                        "sso_unicode": wechat_openid['openid'],
                        "app_id": sub_appid
                    }
                    UserSsoToUser.objects.create(**sso_data)
                # 创建用户登录信息，绑定token
                auth = {
                    'user_id': current_user.get('id', None),
                    'password': make_password('123456', None, 'pbkdf2_sha1'),
                    'plaintext': '123456',
                    'token': token,
                }
                Auth.objects.update_or_create({'user_id': current_user.get('id', None)}, **auth)
                auth_set = Auth.objects.filter(user_id=current_user.get('id', None), password__isnull=False).order_by('-update_time').first()
            else:
                # 用户存在的时候
                current_user = current_user[0]
                sso = UserSsoToUser.objects.filter(user_id=current_user.get('id', None), app_id=sub_appid).first()
                if not sso:
                    if sso_serve_id:
                        sso_data = {
                            "sso_serve_id": sso_serve_id,
                            "user_id": current_user.get('id', None),
                            "sso_unicode": wechat_openid['openid'],
                            "app_id": sub_appid
                        }
                        UserSsoToUser.objects.create(**sso_data)
                token = self.__set_token(current_user.get('id', None), phone)
                # 创建用户登录信息，绑定token
                auth = {
                    'token': token,
                }
                Auth.objects.filter(user_id=current_user.get('id', None)).update(**auth)
                auth_set = Auth.objects.filter(
                    user_id=current_user.get('id', None),
                    password__isnull=False
                ).order_by('-update_time').first()

            # 绑定用户关系 邀请关系和收益关系
            data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=detail_params, user_info=current_user)
            if relate_err:
                logger.error(
                    '绑定用户关系异常：' + str(relate_err) +
                    ' \n当前用户ID:' + str(current_user.get("id", "")) +
                    '\n detail_params:' + json.dumps(detail_params or {})
                )

            return 0, {'token': auth_set.token, 'user_info': current_user}
        except Exception as g_e:
            logger.error('---登录异常程序错误：' + str(g_e) + 'phone:' + str(phone or "") + "line:" + str(g_e.__traceback__.tb_lineno))
            return None, "登录异常"

    def __set_token(self, user_id, account):
        # 生成过期时间
        expire_timestamp = datetime.utcnow() + timedelta(
            days=7,
            seconds=0
        )
        # 返回token
        return jwt.encode(
            payload={'user_id': user_id, 'account': account, "exp": expire_timestamp},
            key=jwt_secret_key
        )

    def get_access_token(self):
        # access_token = self.redis.get('access_token')
        # if access_token:
        #     ttl = self.redis.ttl('access_token')
        #     return {"access_token": access_token.decode('utf-8'), 'expires_in': ttl, 'local': True}
        param = {
            'appid': sub_appid,
            'secret': sub_app_secret,
            'grant_type': 'client_credential'
        }
        response = requests.get(self.wx_token_url, param).json()
        # if 'access_token' in response.keys():
        #     self.redis.set('access_token', response['access_token'])
        #     self.redis.expire('access_token', response['expires_in'])
        return response

    def __del__(self):
        self.redis.close()



# encoding: utf-8
"""
@project: djangoModel->user_relate_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户关系服务
@created_time: 2022/12/13 16:45
"""
from django.core.paginator import Paginator
from django.db.models import F

from xj_role.services.role_service import RoleService
from ..models import UserRelateType, UserRelateToUser, BaseInfo
# 用户关系类型服务
from ..utils.custom_tool import format_params_handle


class UserRelateTypeService():
    @staticmethod
    def list(params=None):
        if params is None:
            params = {}
        size = params.get("size", 10)
        page = params.get("page", 20)
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "relate_key", "relate_name", ]
        )
        relate_obj = UserRelateType.objects.filter(**filter_params).values()
        count = relate_obj.count()
        page_set = Paginator(relate_obj, size).get_page(page)
        return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["relate_key", "relate_name", "description"]
        )
        try:
            relate_obj = UserRelateType.objects.create(**filter_params)
            return {"id": relate_obj.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, update_params=None):
        if update_params is None:
            update_params = {}
        filter_params = format_params_handle(
            param_dict=update_params,
            filter_filed_list=["relate_key", "relate_name", "description"]
        )
        if not pk or not filter_params:
            return None, "没有可修改的数据"
        try:
            relate_obj = UserRelateType.objects.filter(id=pk)
            if not relate_obj:
                return None, "没有可修改的数据"
            relate_obj.update(**filter_params)

            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_obj = UserRelateType.objects.filter(id=pk)
        if not relate_obj:
            return None, None
        try:
            relate_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None


# 用户关系映射服务
class UserRelateToUserService():
    @staticmethod
    def list(params=None):
        if params is None:
            params = {}
        size = params.get("size", 10)
        page = params.get("page", 20)
        try:
            need_pagination = 1 if int(params.get("need_pagination", 1)) else 0
        except Exception as e:
            need_pagination = 1
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type", "user_relate_type_id"]
        )
        relate_user_obj = UserRelateToUser.objects.filter(**filter_params) \
            .annotate(user_name=F("user__user_name")).annotate(with_user_name=F("with_user__user_name")).annotate(user_relate_type_name=F("user_relate_type__relate_name")).values()

        count = relate_user_obj.count()
        if need_pagination:
            page_set = Paginator(relate_user_obj, size).get_page(page)
            return {'count': count, "page": page, "size": size, "list": list(page_set.object_list)}, None
        else:
            return list(relate_user_obj), None

    @staticmethod
    def add(params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type", "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if filter_params.get("user_id", None) is None or filter_params.get("with_user_id", None) is None or filter_params.get("user_relate_type_id", None) is None:
            return None, "参数错误"

        relate_user_obj = UserRelateToUser.objects.filter(
            user_id=filter_params['user_id'],
            with_user=filter_params['with_user_id'],
            user_relate_type_id=filter_params['user_relate_type_id']
        ).first()
        if relate_user_obj:
            return None, "该用户已经绑定，请勿重复绑定"

        try:
            relate_user_obj = UserRelateToUser.objects.create(**filter_params)
            return {"id": relate_user_obj.id}, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def edit(pk=None, params=None):
        if params is None:
            params = {}
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["user", "user_id", "with_user", "with_user_id", "user_relate_type", "user_relate_type_id"],
            alias_dict={"user": 'user_id', "with_user": "with_user_id", "user_relate_type": "user_relate_type_id"}
        )
        if not pk or not params:
            return None, "没有可修改的数据"

        try:
            relate_user_obj = UserRelateToUser.objects.filter(id=pk)
            if not relate_user_obj:
                return None, "没有可修改的数据"
            relate_user_obj.update(**filter_params)
            return None, None
        except Exception as e:
            return None, str(e)

    @staticmethod
    def delete(pk=None):
        if not pk:
            return None, "参数错误"
        relate_user_obj = UserRelateToUser.objects.filter(id=pk)
        if not relate_user_obj:
            return None, None
        try:
            relate_user_obj.delete()
        except Exception as e:
            return None, "删除异常:" + str(e)
        return None, None

    @staticmethod
    def bind_bxtx_relate(params=None, user_info=None):
        """
        镖行天下绑定用户关系服务
        :param detail_params: 参数
        :param user_info: 用户信息
        :return: None,err_msg
        """
        # 绑定用户关系 邀请关系和收益关系
        if user_info is None:
            user_info = {}
        if params is None:
            params = {}

        inviter_id = params.get("inviter_id", None)
        print("邀请人ID：", inviter_id, "当前用户ID:", user_info.get('id', None))
        if not inviter_id:
            return None, "没有传递邀请人ID无法绑定邀请人"

        # 判断是否是一个有效的用户ID
        inviter = BaseInfo.objects.filter(id=inviter_id).first()
        if not inviter:
            return None, "没有该用户"
        data, err = UserRelateToUserService.add(
            {
                "user_id": user_info.get('id', None),
                "with_user_id": inviter_id,
                "user_relate_type_id": 1
            }
        )
        if err:
            return None, err

        # 邀请人不存在受益人，如果邀请人是业务，则绑定的受益人也是该邀请人
        res, err = RoleService.is_this_role(user_id=inviter_id, role_key="BID-SALESMAN")  # 如果是业务人员
        if res:
            data, err = UserRelateToUserService.add(
                {
                    "user_id": user_info.get('id', None),
                    "with_user_id": inviter_id,
                    "user_relate_type_id": 2
                }
            )
            return None, None

        # 查询邀请人的受益人是谁，如果存在则绑定。
        saler = UserRelateToUser.objects.filter(user_id=inviter_id, user_relate_type_id=2).first()
        if saler:
            data, err = UserRelateToUserService.add(
                {
                    "user_id": user_info.get('id', None),
                    "with_user_id": inviter_id,
                    "user_relate_type_id": 2
                }
            )
            return None, None
        return None, None



# encoding: utf-8
"""
@project: hydrology-station-python-4.0->validate
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 表单验证基类
@created_time: 2022/5/25 11:43
"""
import json

from django import forms
from django.core.exceptions import ValidationError


class Validate(forms.Form):
    """检验基类，子类编写规则，调用父类的validate方法"""

    def validate(self):
        """
        request 请求参数验证
        :return {'code': 'err': self.errors}:
        """
        if self.is_valid():
            return True, None
        else:
            error = json.dumps(self.errors)
            error = json.loads(error)
            temp_error = {}
            # 统一展示小写 提示，中文转义回来
            for k, v in error.items():
                temp_error[k.lower()] = v[0]
            return False, temp_error


class Rule:
    """表单验证规则"""

    @staticmethod
    def credit_code(code):
        """企业信用编码"""
        re_patt = "/^[0-9A-HJ-NPQRTUWXY]{2}\d{6}[0-9A-HJ-NPQRTUWXY]{10}$/"
        res = code.match(re_patt)
        if not res:
            raise ValidationError('不是合理有效的企业信用编码')

    @staticmethod
    def chinese_name(code):
        """中文名字"""
        re_patt = "/^(?:[\u4e00-\u9fa5·]{2,16})$/"
        res = code.match(re_patt)
        if not res:
            raise ValidationError('不是合理有效的中文名字')

    @staticmethod
    def plate_number(code):
        """车牌号"""
        re_patt = "/^[京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-HJ-NP-Z](?:((\d{5}[A-HJK])|([A-HJK][A-HJ-NP-Z0-9][0-9]{4}))|[A-HJ-NP-Z0-9]{4}[A-HJ-NP-Z0-9挂学警港澳])$/"
        res = code.match(re_patt)
        if not res:
            raise ValidationError('不是合理有效的 车牌号')

    @staticmethod
    def id_card(code):
        """身份证号"""
        re_patt = "/^[1-9]\d{7}(?:0\d|10|11|12)(?:0[1-9]|[1-2][\d]|30|31)\d{3}$/"
        res = code.match(re_patt)
        if not res:
            raise ValidationError('不是合理有效的 身份证号')


# 案例使用自定义验证方法
def log_unit_id(value):
    res = True
    if not res:
        raise ValidationError('unit_id不存在')


class ExampleValidate(Validate):
    """验证查询表单"""
    region_code = forms.CharField(
        required=True,
        error_messages={
            "required": "行政编码 必填",
        })
    unit_id = forms.IntegerField(  # 自定义方法
        validators=[log_unit_id],
    )



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/6/15 14:14
"""

import json
from logging import getLogger
import random
import sys
import time
from urllib.parse import parse_qs
import uuid

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from rest_framework.request import Request
import xmltodict


def is_number(s):
    """识别任何语言的数字字符串"""
    try:
        float(s)
        return True
    except ValueError:
        pass

    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass

    return False


# json 结果集返回
def parse_json(result):
    if not result is None:
        if type(result) is str:
            try:
                result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
            except Exception as e:
                return result
        elif type(result) is list:
            for index, value in enumerate(result):
                result[index] = parse_json(value)
        elif type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json(v)
    return result


def format_list_handle(param_list, filter_filed_list=None, remove_filed_list=None, alias_dict=None, remove_repeat=True):
    """
    过滤list内容
    :param param_list: 传入 param_list
    :param filter_filed_list: 需要的字段
    :param remove_filed_list: 需要删除的列表
    :param alias_dict: 元素起别名
    :return:param_list： 处理后的 param_list
    """
    if not param_list:
        return param_list
    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_list = [i for i in param_list if i in filter_filed_list]

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_list = [j for j in param_list if not j in remove_filed_list]

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_list = [alias_dict.get(k, k) for k in param_list]

    # 进行去重
    if remove_repeat:
        param_list = list(set(param_list))

    return param_list


# 字段筛选并替换成别名
def format_params_handle(param_dict, filter_filed_list=None, remove_filed_list=None, alias_dict=None, is_remove_null=False):
    """
    字段筛选并替换成别名
    :param param_dict: 参数值
    :param filter_filed_list: 字段白名单
    :param remove_filed_list: 字段黑名单
    :param alias_dict: 表明字典
    :param is_remove_null:  是否把带有None的值移除掉
    :return: param_dict
    """
    # 转换的数据类型不符合，直接返回出去
    if not isinstance(param_dict, dict):
        raise Exception("param_dict 必须是字典格式")

    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if k in filter_filed_list and (not v is None or not is_remove_null)}

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if not k in remove_filed_list and (not v is None or not is_remove_null)}

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_dict = {alias_dict.get(k, k): v for k, v in param_dict.copy().items()}

    return param_dict


# 结果接字段过滤
def filter_result_field(result_list, filter_filed_list=None, remove_filed_list=None, alias_dict=None):
    # 转换的数据类型不符合，直接返回出去
    if not filter_filed_list and not remove_filed_list and not alias_dict:
        return result_list
    result = []
    for item in result_list:
        temp_dict = item
        # 类型判断 过滤字段
        if filter_filed_list and isinstance(filter_filed_list, list):
            temp_dict = {k: v for k, v in temp_dict.copy().items() if k in filter_filed_list}
        # 类型判断， 剔除字段
        if remove_filed_list and isinstance(remove_filed_list, list):
            temp_dict = {k: v for k, v in temp_dict.copy().items() if not k in remove_filed_list}
        # 类型判断 字段转换
        if alias_dict and isinstance(alias_dict, dict):
            temp_dict = {alias_dict.get(k, k): v for k, v in temp_dict.copy().items()}

        if temp_dict:
            result.append(temp_dict)

    return result


# 请求参数解析
def parse_data(request):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
    method = request.method
    if content_type == "text/plain" or method == "GET":
        try:
            body = request.body.decode("utf-8")
            data = json.loads(body)
        except Exception:
            data = request.GET
            if not data:
                data = request.POST
            if not data:
                data = {}
    elif content_type == "application/json":
        return json.loads(request.body)
    elif content_type == "multipart/form-data":
        data = request.POST
    elif content_type == "application/xml":
        try:
            data = xmltodict.parse(request.body)
            return data.get("body") or data.get("data", {})
        except Exception as e:
            data = {}
    elif content_type == "application/x-www-form-urlencoded":
        data = parse_qs(request.body.decode())
        if data:
            data = {k: v[0] for k, v in data.items()}
        else:
            data = {}

    else:
        data = getattr(request, 'data', {})
    return {k: v for k, v in data.items()}


# 请求参数解析
def request_params_wrapper(func):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance, *args, request=request, request_params={}, **kwargs, )

        # 参数解析
        content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
        method = request.method
        # print("content_type:", content_type, "method:", method)
        if content_type == "text/plain" or method == "GET":
            try:
                body = request.body.decode("utf-8")
                data = json.loads(body)
            except Exception:
                data = request.GET
                if not data:
                    data = request.POST
                if not data:
                    data = {}
        elif content_type == "application/json":
            data = json.loads(request.body)
        elif content_type == "multipart/form-data":
            data = request.POST
        elif content_type == "application/xml":
            try:
                data = xmltodict.parse(request.body)
                data = data.get("body") or data.get("data", {})
            except Exception as e:
                data = {}
        elif content_type == "application/x-www-form-urlencoded":
            data = parse_qs(request.body.decode())
            if data:
                data = {k: v[0] for k, v in data.items()}
            else:
                data = {}
        else:
            data = getattr(request, 'data', {})
        # 闭包抛出
        return func(instance, *args, request=request, request_params={k: v for k, v in data.items()}, **kwargs, )

    return wrapper


def deal_equally(total: int, num: int):
    """
    发牌均分，发完截至, 然后打乱顺序
    :param total: 总数
    :param num: 平均分配给这些人
    :return: list
    """
    every_one_jetton = int((total / num))
    overplus_jetton = total % num
    jetton_list = [every_one_jetton for i in range(num)]
    if overplus_jetton == 0:
        return jetton_list
    for index in range(overplus_jetton):
        jetton_list[index] = every_one_jetton + 1
    random.shuffle(jetton_list)
    return jetton_list


def get_short_id(length=8):
    """
    生成随机字符串，2千万次重复两次。
    :param length:
    :return: 随机字符串
    """
    length = 8 if length > 8 else length
    dictionary = [
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
        "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
        "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
    ]
    random_id = str(uuid.uuid4()).replace("-", '')  # 注意这里需要用uuid4
    buffer = []
    for i in range(0, length):
        start = i * 4
        end = i * 4 + 4
        val = int(random_id[start:end], 16)
        buffer.append(dictionary[val % 62])
    return "".join(buffer)


def write_to_log(level="info", prefix="系统异常", content="", err_obj=None):
    """
    写入日志, 注意仅仅支持python3.0以上版本
    :param level: 写入错误日志等级
    :param prefix: 提示错误类型
    :param content: 错误内容
    :param err_obj: try except 捕捉到的错误对象, 自动处理日志格式
    :return: data, err_msg
    """
    logger = getLogger('log')
    try:
        if not err_obj is None:
            logger.error(
                '---' + prefix + ":" + str(err_obj) + ";" +
                (" content:" + str(content) + ";" if content else "") +
                " line:" + str(err_obj.__traceback__.tb_lineno) + ";" +
                " file:" + str(err_obj.__traceback__.tb_frame.f_globals["__file__"]) + ";" +
                " datetime:" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())) + ";"
            )

        if level == "info":
            logger.error('---' + prefix + ":" + str(content))
        elif level == "error":
            logger.error('---' + prefix + ":" + str(content))
        return True, None
    except Exception as err:
        return False, str(err)


def flow_service_wrapper(func):
    """
    API 流程中间件装饰器
    PS 该装饰器必须配套request_params_wrapper装饰一起使用
    """
    if not sys.modules.get("xj_flow.services.flow_process_service.FlowProcessService"):
        from xj_flow.services.flow_process_service import FlowProcessService

    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, request_params=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        :param request_params: 请求参数解析
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request_params is None:
            request_params = {}

        flow_node_id = request_params.pop("flow_node_id", None)
        flow_action_id = request_params.pop("flow_action_id", None)
        if not flow_node_id or not flow_action_id:
            return func(instance, *args, request=request, request_params=request_params, **kwargs, )

        service = FlowProcessService()
        data, err = service.do_once_flow_in_service(flow_node_id, flow_action_id, source_params=request_params)
        if err:
            write_to_log(prefix="流程装饰器调用异常:", content=err)

        request_params = data.get("source_params", request_params)
        return func(instance, *args, request=request, request_params=request_params, **kwargs, )

    return wrapper
